--------------------------------------------------------------------------------
-- Touhin!
--

local _, Touhin = ...
LibStub("AceAddon-3.0"):NewAddon(Touhin, "Touhin", "AceEvent-3.0")

-- Debug
_G.Touhin = Touhin


--------------------------------------------------------------------------------
-- Locals
--

local media = LibStub("LibSharedMedia-3.0", true)
if media then 
	media:Register("statusbar", "TouhinDark", [[Interface\AddOns\Touhin\TouhinDark]])
end
local deformat = LibStub("LibDeformat-3.0").Deformat

local db = nil
local anchor = nil
local pName = UnitName("player")
local CLASS_COLORS = {}

--temp locale
local L_NOBODY = "Nobody"
local DISENCHANT = ROLL_DISENCHANT -- >.> don't you judge me


--------------------------------------------------------------------------------
-- Anchor
--

local function CreateAnchor()
	local f = CreateFrame("Frame", nil, UIParent)
	f:SetWidth(220)
	f:SetHeight(18)
	f:ClearAllPoints()
	f:SetPoint(db.anchor_point or "CENTER", db.anchor_x or 0, db.anchor_y or 0)
	
	local texture = f:CreateTexture(nil, "BACKGROUND")
	texture:SetAllPoints(f)
	texture:SetBlendMode("BLEND")
	texture:SetTexture(0, 0, 0, 0.3)
	
	local text = f:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	text:SetText("Touhin")
	text:SetAllPoints(f)
	
	f:EnableMouse(true)
	f:SetClampedToScreen(true)
	f:SetScript("OnMouseUp", function(self, button)
		if button == "RightButton" then
			local point, _, _, x, y = self:GetPoint(1)
			db.anchor_point = point
			db.anchor_x = x
			db.anchor_y = y
			self:Hide()
		elseif button == "LeftButton" and IsShiftKeyDown() then
			Touhin:AddLootTest()
		elseif button == "MiddleButton" then
			Touhin:OpenOptions()
		end
	end)
	
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", f.StartMoving)
	f:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		local point, _, _, x, y = self:GetPoint(1)
		db.anchor_point = point
		db.anchor_x = x
		db.anchor_y = y
	end)
	
	f:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT", 0, 4)
		GameTooltip:ClearLines()
		GameTooltip:AddLine("Touhin")
		GameTooltip:AddLine("|cffeda55fDrag|r to position the anchor.\n" ..
			"|cffeda55fShift-Click|r to spawn a test loot frame.\n" ..
			"|cffeda55fRight-Click|r to hide the anchor.\n" ..
			"|cffeda55fMiddle-Click|r to open options.\n", 0.2, 1, 0.2, 1) --TODO localize
		GameTooltip:Show()
	end)
	f:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	
	f:Hide()
	
	anchor = f
end

local function ShowAnchor()
	if not anchor then CreateAnchor() end
	anchor:Show()
end

local function HideAnchor()
	if not anchor then return end
	anchor:Hide()
end

local function UpdateAnchor()
	if not anchor then return end
	anchor:ClearAllPoints()
	anchor:SetPoint(db.anchor_point or "CENTER", db.anchor_x or 0, db.anchor_y or 0)
end

local function ResetAnchor()
	if anchor then
		anchor:ClearAllPoints()
		anchor:SetPoint("CENTER")
	end
	db.anchor_point = nil
	db.anchor_x = nil
	db.anchor_y = nil
end

local function ToggleAnchor()
	if not anchor or not anchor:IsShown() then
		ShowAnchor()
	else
		HideAnchor()
	end
end


--------------------------------------------------------------------------------
-- Frame stacking
--

local UpdateRows, AddRow, RemoveRow
do
	local function sorter(a, b)
		return a.time < b.time and true or false
	end
	
	local rows = {}
	local tmp = {}
	
	-- FIXME the text seems to "bounce" up one slot when arranging :\ anchoring issue? no idea z.z
	-- removal flow: (Alpha_OnFinished) -> RemoveRow -> Hide -> UpdateRows -> ReleaseFrame
	function UpdateRows()
		wipe(tmp)
		for row in next, rows do
			tmp[#tmp + 1] = row
		end
		table.sort(tmp, sorter)
		
		local last = anchor
		for i=1, db.shownLimit do
			if not tmp[i] then break end
			tmp[i]:ClearAllPoints()
			if db.growDown then
				tmp[i]:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, -2)
			else
				tmp[i]:SetPoint("BOTTOMLEFT", last, "TOPLEFT", 0, 2)
			end
			last = tmp[i]
		end
	end

	function AddRow(row)
		if not anchor then CreateAnchor() end
		rows[row] = true
		row:Show()
		UpdateRows()
	end

	function RemoveRow(row)
		rows[row] = nil
		row:Hide()
		UpdateRows()
		--[[--doesn't fix the issue, although it is probably more efficient, lol
		for i=1,#tmp do
			if tmp[i] == row then
				tremove(tmp, i)
				if tmp[i] then
					if db.growDown then
						tmp[i]:SetPoint("TOPLEFT", tmp[i-1] or anchor, "BOTTOMLEFT", 0, -2)
					else
						tmp[i]:SetPoint("BOTTOMLEFT", tmp[i-1] or anchor, "TOPLEFT", 0, 2)
					end
				end
				break
			end
		end--]]
	end
end


--------------------------------------------------------------------------------
-- Frame creation
--

local GetFrame, ReleaseFrame, SetFrameBackdrop
do
	--scripts
	local function frame_OnShow(self)
		self.fader:GetParent():Play()
	end
	
	local function frame_OnEnter(self)
		if not self.link then return end
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetHyperlink(self.link)
	end
	
	local function frame_OnLeave(self)
		if not self.link then return end
		GameTooltip:Hide()
	end
	
	local function frame_OnClick(self, button)
		self.fader:GetParent():Stop()
	end
	
	local function fader_OnFinished(self, requested)
		RemoveRow(self.object)
		ReleaseFrame(self.object)
	end
	
	local backdrop = {
		bgFile = "Interface\\AddOns\\Touhin\\TouhinDark", --"Interface\\ChatFrame\\ChatFrameBackground"
		tile = true, tileSize = 22,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
		insets = {left = 4, right = 4, top = 4, bottom = 4},
	}
	
	--sharedmedianess
	function SetFrameBackdrop(bgFile, edgeFile, edgeSize, inset)
		if not media then return end
		
		if media:Fetch("statusbar", bgFile, true) then
			backdrop.bgFile = media:Fetch("statusbar", bgFile)
		end
		if media:Fetch("border", edgeFile, true) then
			backdrop.edgeFile = media:Fetch("border", edgeFile)
		end
		if edgeSize then
			backdrop.edgeSize = edgeSize
		end
		if inset then
			backdrop.insets.left = inset
			backdrop.insets.right = inset
			backdrop.insets.top = inset
			backdrop.insets.bottom = inset
		end
	end
	
	--recycling
	local frameHeap = {}
	
	function GetFrame()
		local frame = tremove(frameHeap)
		--print("GetFrame", frame)
		if not frame then
			--shiney new frame
			frame = CreateFrame("Button", nil, UIParent)
			frame:SetWidth(220)
			frame:SetHeight(22)
			frame:SetBackdrop(backdrop)
			frame:SetBackdropColor(0, 0, 0, .9)
			frame:Hide()
			
			--[[local texture = frame:CreateTexture(nil, "BORDER")
			texture:SetPoint("TOPLEFT", frame, "TOPLEFT", 4, -4)
			texture:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -4, 4)
			texture:SetTexture(background)
			frame.background = texture--]]
			
			local iconFrame = CreateFrame("Frame", nil, frame)
			iconFrame:SetPoint("LEFT", 5, 0)
			iconFrame:SetWidth(26)
			iconFrame:SetHeight(26)
			iconFrame:SetBackdrop(backdrop)
			iconFrame:SetBackdropColor(1, 1, 1, 0)
			frame.iconFrame = iconFrame
			
			local icon = iconFrame:CreateTexture(nil, "OVERLAY")
			icon:SetPoint("TOPLEFT", iconFrame, "TOPLEFT", 4, -4)
			icon:SetPoint("BOTTOMRIGHT", iconFrame, "BOTTOMRIGHT", -4, 4)
			icon:SetTexCoord(.07, .93, .07, .93) --0.1, 0.9, 0.1, 0.9
			frame.icon = icon
			
			local text = frame:CreateFontString(nil, "OVERLAY")
			text:SetPoint("LEFT", icon, "RIGHT", 5, 0)
			text:SetPoint("RIGHT", frame, -5, 0)
			text:SetFont("Fonts\\FRIZQT__.TTF", 10, "")
			text:SetJustifyH("LEFT")
			text:SetJustifyV("MIDDLE")
			text:SetTextColor(1, 1, 1, 1)
			text:SetShadowOffset(0.8, -0.8)
			text:SetShadowColor(0, 0, 0, 1)
			frame.text = text
			
			--[[floating or on the frame between icon and text, hmhm, thinking on the bar ;[
			local rollIcon = CreateFrame("Button", nil, frame)
			rollIcon:SetPoint("LEFT", frame, "RIGHT", -2, -2)
			rollIcon:SetWidth(24)
			rollIcon:SetHeight(24)
			--rollIcon:SetScript("OnEnter", roll_OnEnter)
			--rollIcon:SetScript("OnLeave", roll_OnLeave)
			frame.rollIcon = rollIcon--]]
			
			--a timer and fancy fading in one, pretty neat
			local ag = frame:CreateAnimationGroup()
			ag:SetLooping("NONE")
			
			local fader = ag:CreateAnimation("Alpha")
			fader:SetChange(-1)
			fader:SetStartDelay(db.fadeDelay)
			--fader:SetEndDelay(0.1)
			fader:SetDuration(db.fadeTime)
			fader:SetSmoothing("IN")
			fader:SetScript("OnStop", fader_OnFinished)
			fader:SetScript("OnFinished", fader_OnFinished)
			fader:SetScript("OnUpdate", nil)
			fader.object = frame
			frame.fader = fader
			
			frame:SetScript("OnShow", frame_OnShow)
			frame:SetScript("OnEnter", frame_OnEnter)
			frame:SetScript("OnLeave", frame_OnLeave)
			frame:SetScript("OnClick", frame_OnClick)
			frame:RegisterForClicks("RightButtonUp")
		else
			frame:SetAlpha(1)
			frame:SetBackdrop(backdrop)
			frame:SetBackdropColor(0, 0, 0, .9)
			frame.iconFrame:SetBackdrop(backdrop)
			frame.iconFrame:SetBackdropColor(1, 1, 1, 0)
			frame.fader:SetStartDelay(db.fadeDelay)
			frame.fader:SetDuration(db.fadeTime)
		end
		
		return frame
	end

	function ReleaseFrame(frame)
		--print("ReleaseFrame", frame.fader:IsStopped() or frame.fader:IsDone())
		frame:Hide()
		frame:ClearAllPoints()
		--frame:SetBackdrop(nil)
		--frame.iconFrame:SetBackdrop(nil)
		tinsert(frameHeap, frame)
	end
end


--------------------------------------------------------------------------------
-- Parse chat messages
--

--[[
TODO! Roll messages

LOOT_ROLL_DISENCHANT_SELF = "You have selected Disenchant for: %s";
LOOT_ROLL_GREED_SELF = "You have selected Greed for: %s";
LOOT_ROLL_NEED_SELF = "You have selected Need for: %s";
LOOT_ROLL_DISENCHANT = "%s has selected Disenchant for: %s";
LOOT_ROLL_GREED = "%s has selected Greed for: %s";
LOOT_ROLL_NEED = "%s has selected Need for: %s";
L_ROLL = "(.+) has?v?e? selected (.+) for: (.+)"

LOOT_ROLL_ROLLED_DE = "Disenchant Roll - %d for %s by %s";
LOOT_ROLL_ROLLED_GREED = "Greed Roll - %d for %s by %s";
LOOT_ROLL_ROLLED_NEED = "Need Roll - %d for %s by %s";
L_ROLLED = "(.+) Roll . (%d+) for (.+) by (.+)"

LOOT_ROLL_WON = "%s won: %s";
LOOT_ROLL_YOU_WON = "You won: %s";
L_WON = "(.+) won: (.+)"

Detailed Loot Information off?
LOOT_ROLL_YOU_WON_NO_SPAM_DE = "You won: %2$s |cff818181(Disenchant - %1$d)|r";
LOOT_ROLL_YOU_WON_NO_SPAM_GREED = "You won: %2$s |cff818181(Greed - %1$d)|r";
LOOT_ROLL_YOU_WON_NO_SPAM_NEED = "You won: %2$s |cff818181(Need - %1$d)|r";
LOOT_ROLL_WON_NO_SPAM_DE = "%1$s won: %3$s |cff818181(Disenchant - %2$d)|r";
LOOT_ROLL_WON_NO_SPAM_GREED = "%1$s won: %3$s |cff818181(Greed - %2$d)|r";
LOOT_ROLL_WON_NO_SPAM_NEED = "%1$s won: %3$s |cff818181(Need - %2$d)|r";
L_WON_NOSPAM = "(.+) won: (.+) |cff818181.(.+) . (%d+).|r";
--]]
local ParseRollMessage, GetLootRoll
do
	local rolls = {}
	
	function ParseRollMessage(msg)
	end

	function GetLootRoll(item, player)
		for i,v in ipairs(rolls) do
			if v.item == item and v.winner == player then
				return tremove(rolls, i)
			end
		end
	end
end

--[[
LOOT_ITEM = "%s receives loot: %s.";
LOOT_ITEM_MULTIPLE = "%s receives loot: %sx%d.";
LOOT_ITEM_SELF = "You receive loot: %s.";
LOOT_ITEM_SELF_MULTIPLE = "You receive loot: %sx%d.";
LOOT_ROLL_ALL_PASSED = "Everyone passed on: %s";
--]]
local function ParseLootMessage(msg)
	local player = pName
	local item, quantity = deformat(msg, LOOT_ITEM_SELF_MULTIPLE)
	if quantity then
		return player, item, tonumber(quantity)
	end
	
	item = deformat(msg, LOOT_ITEM_SELF)
	if item then
		return player, item, 1
	end

	player, item, quantity = deformat(msg, LOOT_ITEM_MULTIPLE)
	if quantity then
		return player, item, tonumber(quantity)
	end
	
	player, item = deformat(msg, LOOT_ITEM)
	if item then
		return player, item, 1
	end
	
	item = deformat(msg, LOOT_ROLL_ALL_PASSED)
	if item then
		return L_NOBODY, item, 0
	end
end

function Touhin:CHAT_MSG_LOOT(_, msg)
	local player, item, quantity = ParseLootMessage(msg)
	--print("CHAT_MSG_LOOT", player, item, quantity)
	if player and item and quantity then
		self:AddLoot(player, item, quantity)
	end
end


--[[
LOOT_MONEY = "%s loots %s.";
LOOT_MONEY_SPLIT = "Your share of the loot is %s.";
TODO (cata) LOOT_MONEY_SPLIT_GUILD = "Your share of the loot is %s. (%s deposited to guild bank)";
--]]
local ParseMoneyMessage
do
	-- Parse coin strings (code from SpecialEventsEmbed-Loot)
	local cg = GOLD_AMOUNT:gsub("%%d", "(%1+)")
	local cs = SILVER_AMOUNT:gsub("%%d", "(%1+)")
	local cc = COPPER_AMOUNT:gsub("%%d", "(%1+)")
	
	local function parseCoinString(str)
		local g = string.match(str, cg) or 0
		local s = string.match(str, cs) or 0
		local c = string.match(str, cc) or 0
		return g * 10000 + s * 100 + c
	end
	
	function ParseMoneyMessage(msg)
		local player = pName
		local money = deformat(msg, LOOT_MONEY_SPLIT)
		if money then
			return player, parseCoinString(money)
		end
		
		money = deformat(msg, YOU_LOOT_MONEY)
		if money then
			return player, parseCoinString(money)
		end

		player, money = deformat(msg, LOOT_MONEY)
		if player and money then
			return player, parseCoinString(money)
		end
	end
end

function Touhin:CHAT_MSG_MONEY(_, msg)
	local player, quantity = ParseMoneyMessage(msg)
	if player and quantity then
		self:AddLoot(player, "coin", quantity)
	end
end


--------------------------------------------------------------------------------
-- Show Loot!
--

function Touhin:AddLootTest()
	if math.random() > .2 then
		local item
		repeat
			item = GetContainerItemLink(math.random(0, 4), math.random(1, 22))
		until item
		self:AddLoot(pName, item, 0)
	else
		self:AddLoot(pName, "coin", math.random(1, 50000))
	end
end

local function GetCoinString(amount)
	local s = GetCoinText(amount, " ")
	s = string.gsub(s, "(%d+) Gold", "|cffffffff%1|r|cffffd700g|r")
	s = string.gsub(s, "(%d+) Silver", "|cffffffff%1|r|cffc7c7cfs|r")
	s = string.gsub(s, "(%d+) Copper", "|cffffffff%1|r|cffeda55fc|r")
	return s
end

local rollTextures = {
	[1] = "Interface\\Buttons\\UI-GroupLoot-Dice-Up",
	[2] = "Interface\\Buttons\\UI-GroupLoot-Coin-Up",
	[3] = "Interface\\Buttons\\UI-GroupLoot-DE-Up",
	[4] = "Interface\\Buttons\\UI-GroupLoot-Pass-Up",
	[NEED] = "Interface\\Buttons\\UI-GroupLoot-Dice-Up",
	[GREED] = "Interface\\Buttons\\UI-GroupLoot-Coin-Up",
	[DISENCHANT] = "Interface\\Buttons\\UI-GroupLoot-DE-Up",
	[PASS] = "Interface\\Buttons\\UI-GroupLoot-Pass-Up",
}

function Touhin:AddLoot(unitName, item, itemCount)
	local itemName, itemLink, itemQuality, itemTexture
	if item ~= "coin" then
		itemName, itemLink, itemQuality, _, _, _, _, _, _, itemTexture = GetItemInfo(item)
		if not itemName then return end
		
		local own = UnitIsUnit("player", unitName)
		if (own and itemQuality < db.qualitySelfThreshold) or (not own and itemQuality < db.qualityThreshold) then
			return
		end
		if own and db.showCount then
			itemCount = itemCount + GetItemCount(itemLink) or 0
		end
		if unitName == L_NOBODY then
			itemTexture = rollTextures[PASS]
		end
	else
		if not db.showMoney then return end
		itemName = "coin"
		itemQuality = 0
		itemTexture = GetCoinIcon(itemCount)
	end
	
	--print("AddLoot", unitName, itemName, itemCount)
	local row = GetFrame()
	row.link = itemLink
	row.unit = unitName
	
	local qualityColor = ITEM_QUALITY_COLORS[itemQuality]
	
	if item == "coin" then
		row.text:SetText(GetCoinString(itemCount))
		row.time = 0 - itemCount --money always on top (giggity)
	else
		if string.len(itemName) > db.displayLength then
			itemName = string.sub(itemName, 1, db.displayLength) .. ".."
		end
		local classColor = CLASS_COLORS[select(2, UnitClass(unitName)) or false] or {r = 0.8, g = 0.8, b = 0.8}
		classColor = string.format("|cff%02x%02x%02x", classColor.r * 255, classColor.g * 255, classColor.b * 255)
		
		row.text:SetFormattedText("%s%s|r %s[%s]|r |cffaaaaaa%s|r", classColor, unitName, qualityColor.hex, itemName, itemCount > 1 and itemCount or "")
		row.time = GetTime()
	end
	
	--[[local roll = GetLootRoll(unitName, itemLink) or true
	if roll then
		--row.rollIcon.tooltip = roll.info
		row.rollIcon:SetNormalTexture(rollTextures[math.random(1,3)])
		row.rollIcon:Show()
	else
		row.rollIcon:Hide()
	end--]]
	
	row:SetBackdropBorderColor(qualityColor.r, qualityColor.g, qualityColor.b)
	row:SetBackdropColor(qualityColor.r, qualityColor.g, qualityColor.b)
	row.iconFrame:SetBackdropBorderColor(qualityColor.r, qualityColor.g, qualityColor.b)
	row.icon:SetTexture(itemTexture)
	
	row:SetWidth(4 + 5 + 20 + 5 + row.text:GetStringWidth() + 5) -- insets + pad + icon + pad + text + pad = text + 39
	
	AddRow(row)
end


--------------------------------------------------------------------------------
-- Options
--

local defaults = {
	profile = {
		showMoney = true,
		showLoot = true,
		showCount = true,
		showRollWinner = true,
		showRollChoices = true,
		qualityThreshold = 3,
		qualitySelfThreshold = 0,
		displayLength = 100,
		growDown = true,
		shownLimit = 10,
		fadeDelay = 8,
		fadeTime = .4,
	}
}

local function GetOptions()
	local options = {
		name = "Touhin",
		type = "group",
		get = function(info) return db[info[#info]] end,
		set = function(info, value) db[info[#info]] = value end,
		args = {
			header = {
				name = GetAddOnMetadata("Touhin", "Notes").."\n",
				type = "description",
				order = 1,
				width = "full",
			},
			toggleAnchor = {
				name = "toggleAnchor",
				desc = "toggleAnchor",
				type = "execute",
				func = ToggleAnchor,
				order = 2,
			},
			resetAnchor = {
				name = "resetAnchor",
				desc = "resetAnchor",
				type = "execute",
				func = ResetAnchor,
				order = 3,
			},
			growDown = {
				name = "growDown",
				desc = "growDown",
				type = "toggle",
				set = function(info, value)
					db[info[#info]] = value
					UpdateRows()
				end,
				order = 4,
			},
			conditions = {
				name = "conditions",
				type = "group",
				inline = true,
				order = 10,
				args = {
					showMoney = {
						name = "showMoney",
						desc = "showMoney",
						type = "toggle",
						order = 1,
					},
					showCount = {
						name = "showCount",
						desc = "showCount",
						type = "toggle",
						order = 2,
					},
					qualityThreshold = {
						name = "qualityThreshold",
						desc = "qualityThreshold",
						type = "select",
						values = {
							[0] = ITEM_QUALITY0_DESC,
							[1] = ITEM_QUALITY1_DESC,
							[2] = ITEM_QUALITY2_DESC,
							[3] = ITEM_QUALITY3_DESC,
							[4] = ITEM_QUALITY4_DESC,
							[5] = ITEM_QUALITY5_DESC
						},
						order = 3,
					},
					qualitySelfThreshold = {
						name = "qualitySelfThreshold",
						desc = "qualitySelfThreshold",
						type = "select",
						values = {
							[0] = ITEM_QUALITY0_DESC,
							[1] = ITEM_QUALITY1_DESC,
							[2] = ITEM_QUALITY2_DESC,
							[3] = ITEM_QUALITY3_DESC,
							[4] = ITEM_QUALITY4_DESC,
							[5] = ITEM_QUALITY5_DESC
						},
						order = 4,
					},
				},
			},
			display = {
				name = "display",
				type = "group",
				inline = true,
				order = 20,
				args = {
					displayLength = {
						name = "displayLength",
						desc = "displayLength",
						type = "range", min = 1, max = 300, softMax = 100, step = 1,
						order = 1,
					},
					shownLimit = {
						name = "shownLimit",
						desc = "shownLimit",
						type = "range", min = 1, max = 25, step = 1,
						order = 2,
					},
					fadeDelay = {
						name = "fadeDelay",
						desc = "fadeDelay",
						type = "range", min = 1, max = 30, step = 1,
						order = 3,
					},
					fadeTime = {
						name = "fadeTime",
						desc = "fadeTime",
						type = "range", min = .1, max = 1, step = .1,
						order = 4,
					},
				},
			},
		},
	}
	return options
end

function Touhin:OpenOptions()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end


--------------------------------------------------------------------------------
-- Initialization
--

function Touhin:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("TouhinDB", defaults, true)
	
	local function OnProfileChanged()
		db = self.db.profile
		UpdateAnchor()
	end
	
	self.db.RegisterCallback(self, "OnProfileChanged", OnProfileChanged)
	self.db.RegisterCallback(self, "OnProfileCopied", OnProfileChanged)
	self.db.RegisterCallback(self, "OnProfileReset", OnProfileChanged)
	db = self.db.profile
	
	SetFrameBackdrop(db.bgFile, db.edgeFile, db.edgeSize, db.insets)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Touhin", GetOptions)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Touhin")
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Touhin Profile", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db))
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Touhin Profile", "Profiles", "Touhin")
end

function Touhin:OnEnable()
	if not db.anchor_point then
		ShowAnchor()
	end
	
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("CHAT_MSG_MONEY")
	
	--!ClassColors support
	if CUSTOM_CLASS_COLORS then
		local function updateClassColors()
			for k,v in next, CUSTOM_CLASS_COLORS do
				CLASS_COLORS[k] = v
			end
		end
		CUSTOM_CLASS_COLORS:RegisterCallback(updateClassColors)
		updateClassColors()
	else
		CLASS_COLORS = RAID_CLASS_COLORS
	end
end

SLASH_TOUHIN1 = "/touhin"
SlashCmdList["TOUHIN"] = function(input)
	ToggleAnchor()
end
